#if !defined(LEXER_HH)
#define LEXER_HH

#include <iostream>
#include <string>

using std::string;

#include "Token.hh"
#include "ErrorManager.hh"

///////////////////////////////////////////////////////////////////////////
class Lexer {
///////////////////////////////////////////////////////////////////////////

public :

  Lexer( std::istream& is, ErrorManager& error ) : 
    inStream_(is), error_(error) { }
  ~Lexer( void ) { }

  void next() {
    char ch;
    while (inStream_.get(ch) && isLayout_(ch));
    if (!inStream_.eof()) {
      if (isDigit_(ch)) {
	token_.setPartOfSpeech(Token::NUMBER);
	string tmp(1,ch);
	while (inStream_.get(ch) && isDigit_(ch)) {
	  tmp.push_back(ch);
	}
	inStream_.putback(ch);
	token_.setLexeme(tmp);
      } else {
	string tmp(1,ch);
	token_.setLexeme(tmp);
	switch(ch) {
	case '(' : token_.setPartOfSpeech(Token::LPAR); break;
	case ')' : token_.setPartOfSpeech(Token::RPAR); break;
	case '+' : 
	case '-' : 
	case '*' : 
	case '/' : 
	  token_.setPartOfSpeech(Token::OPERATOR); break;
	default : 
	  token_.setPartOfSpeech(Token::NIL);
	}
      }
    } else {
      token_.setLexeme(string(""));
      token_.setPartOfSpeech(Token::EOFF);
    }
  }

  const Token& cur() {
    return token_;
  }

private :
  std::istream& inStream_;
  ErrorManager& error_;
  Token         token_;

  bool isLayout_( char ch ) {
    switch (ch) {
    case ' ': case '\t' : case '\n': return true;
    default : return false;
    }
  }
  bool isDigit_( char ch ) {
    return (ch >= '0' && ch <= '9');
  } 

}; // End of class Lexer

#endif
